@@ -4,7 +4,7 @@ package PONAPI::Server;
 use strict;
 use warnings;
 
-our $VERSION = '0.002012';
+our $VERSION = '0.002013';
 
 use Plack::Request;
 use Plack::Response;
@@ -61,12 +61,15 @@ sub call {
         $self->_ponapi_params( shift, $req )
     };
 
+    return $self->_options_response( $ponapi_params->{__options__} )
+        if $ponapi_params->{__options__};
+
     return $self->_error_response( $ponapi_params->{__error__} )
         if $ponapi_params->{__error__};
 
     my $action = delete $ponapi_params->{action};
     my ( $status, $headers, $res ) = $self->{'ponapi.DAO'}->$action($ponapi_params);
-    return $self->_response( $status, $headers, $res );
+    return $self->_response( $status, $headers, $res, $req->method eq 'HEAD' );
 }
 
 
@@ -85,6 +88,12 @@ sub _load_dao {
     );
 }
 
+sub _is_get_like {
+    my $req = shift;
+    return 1 if $req->method =~ /^(?:GET|HEAD)$/;
+    return;
+}
+
 sub _ponapi_params {
     my ( $self, $wr, $req ) = @_;
 
@@ -104,7 +113,7 @@ sub _ponapi_params {
     my $req_base      = $self->{'ponapi.relative_links'} eq 'full' ? "".$req->base : '/';
     my $req_path      = $self->{'ponapi.relative_links'} eq 'full' ? "".$req->uri : $req->request_uri;
     my $update_200    = !!$self->{'ponapi.respond_to_updates_with_200'};
-    my $doc_self_link = ($req->method eq 'GET') ? !!$self->{'ponapi.doc_auto_self_link'} : 0;
+    my $doc_self_link = _is_get_like($req) ? !!$self->{'ponapi.doc_auto_self_link'} : 0;
 
     my %params = (
         @ponapi_route_params,
@@ -123,7 +132,7 @@ sub _ponapi_route_match {
     my ( $self, $wr, $req ) = @_;
     my $method = $req->method;
 
-    $wr->(ERR_BAD_REQ) unless grep { $_ eq $method } qw< GET POST PATCH DELETE >;
+    $wr->(ERR_BAD_REQ) unless grep { $_ eq $method } qw< GET POST PATCH DELETE HEAD OPTIONS >;
 
     my ( $type, $id, $relationships, $rel_type ) = split '/' => substr($req->path_info,1);
 
@@ -147,19 +156,30 @@ sub _ponapi_route_match {
     my $action;
     if ( defined $id ) {
         $action = 'create_relationships'     if $method eq 'POST'   and $relationships  and $def_rel_type;
-        $action = 'retrieve'                 if $method eq 'GET'    and !$relationships and !$def_rel_type;
-        $action = 'retrieve_by_relationship' if $method eq 'GET'    and !$relationships and $def_rel_type;
-        $action = 'retrieve_relationships'   if $method eq 'GET'    and $relationships  and $def_rel_type;
+        $action = 'retrieve'                 if _is_get_like($req)  and !$relationships and !$def_rel_type;
+        $action = 'retrieve_by_relationship' if _is_get_like($req)  and !$relationships and $def_rel_type;
+        $action = 'retrieve_relationships'   if _is_get_like($req)  and $relationships  and $def_rel_type;
         $action = 'update'                   if $method eq 'PATCH'  and !$relationships and !$def_rel_type;
         $action = 'update_relationships'     if $method eq 'PATCH'  and $relationships  and $def_rel_type;
         $action = 'delete'                   if $method eq 'DELETE' and !$relationships and !$def_rel_type;
         $action = 'delete_relationships'     if $method eq 'DELETE' and $relationships  and $def_rel_type;
     }
     else {
-        $action = 'retrieve_all'             if $method eq 'GET';
+        $action = 'retrieve_all'             if _is_get_like($req);
         $action = 'create'                   if $method eq 'POST';
     }
 
+    if ( $method eq 'OPTIONS' ) {
+        my @options = (qw< GET HEAD > );
+        if ( defined $id ) {
+            push @options => (qw< PATCH DELETE >) unless $def_rel_type;
+        }
+        else {
+            push @options => 'POST';
+        }
+        $wr->( +{ __options__ => \@options } );
+    }
+
     $wr->(ERR_NO_MATCHING_ROUTE) unless $action;
 
     # return ( action, type, id?, rel_type? )
@@ -172,6 +192,8 @@ sub _ponapi_route_match {
 sub _ponapi_check_headers {
     my ( $self, $wr, $req ) = @_;
 
+    return if $req->method eq 'OPTIONS';
+
     my $pack = HTTP::Headers::ActionPack->new;
     my $mt   = $self->{'ponapi.mediatype'};
 
@@ -261,7 +283,7 @@ sub _ponapi_data {
 
     return unless $req->content_length and $req->content_length > 0;
 
-    $wr->(ERR_BAD_REQ) if $req->method eq 'GET';
+    $wr->(ERR_BAD_REQ) if _is_get_like($req);
 
     my $body;
     eval { $body = JSON::XS::decode_json( $req->content ); 1 };
@@ -327,7 +349,7 @@ sub _validate_data_members {
 }
 
 sub _response {
-    my ( $self, $status, $headers, $content ) = @_;
+    my ( $self, $status, $headers, $content, $is_head ) = @_;
     my $res = Plack::Response->new( $status || 200 );
 
     $res->headers($headers);
@@ -336,12 +358,17 @@ sub _response {
         if $self->{'ponapi.send_version_header'};
     if ( ref $content ) {
         my $enc_content = JSON::XS::encode_json $content;
-        $res->content($enc_content);
         $res->content_length( length($enc_content) );
+        $res->content($enc_content) unless $is_head;
     }
     $res->finalize;
 }
 
+sub _options_response {
+    my ( $self, $options ) = @_;
+    return +[ 200, [ Allow => join( ', ' => @{$options} ) ], [] ];
+}
+
 sub _error_response {
     my ( $self, $args ) = @_;
 
@@ -365,7 +392,7 @@ PONAPI::Server - PONAPI - Perl implementation of {JSON:API} (http://jsonapi.org/
 
 =head1 VERSION
 
-version 0.002012
+version 0.002013
 
 =head1 SYNOPSIS
 
